@@ -1,15 +1,14 @@
 var PETRA = function(m)
 {
 
-/* Naval Manager
- Will deal with anything ships.
- -Basically trade over water (with fleets and goals commissioned by the economy manager)
- -Defense over water (commissioned by the defense manager)
-	-subtask being patrols, escort, naval superiority.
- -Transport of units over water (a few units).
- -Scouting, ultimately.
- Also deals with handling docks, making sure we have access and stuffs like that.
- Does not build them though, that's for the base manager to handle.
+/**
+ * Naval Manager
+ * Will deal with anything ships.
+ * -Basically trade over water (with fleets and goals commissioned by the economy manager)
+ * -Defense over water (commissioned by the defense manager)
+ * -Transport of units over water (a few units).
+ * -Scouting, ultimately.
+ * Also deals with handling docks, making sure we have access and stuffs like that.
  */
 
 m.NavalManager = function(Config)
@@ -36,7 +35,7 @@ m.NavalManager = function(Config)
 	this.landingZones = {};
 };
 
-// More initialisation for stuff that needs the gameState
+/** More initialisation for stuff that needs the gameState */
 m.NavalManager.prototype.init = function(gameState, deserializing)
 {
 	// finished docks
@@ -55,8 +54,8 @@ m.NavalManager.prototype.init = function(gameState, deserializing)
 	this.warShips.registerUpdates();
 	this.fishShips.registerUpdates();
 	
-	var fishes = gameState.getFishableSupplies();
-	var availableFishes = {};
+	let fishes = gameState.getFishableSupplies();
+	let availableFishes = {};
 	for (let fish of fishes.values())
 	{
 		let sea = gameState.ai.accessibility.getAccessValue(fish.position(), true);
@@ -121,8 +120,8 @@ m.NavalManager.prototype.init = function(gameState, deserializing)
 		return;
 
 	// determination of the possible landing zones
-	var width = gameState.getMap().width;
-	var length = width * gameState.getMap().height;
+	let width = gameState.getMap().width;
+	let length = width * gameState.getMap().height;
 	for (let i = 0; i < length; ++i)
 	{
 		let land = gameState.ai.accessibility.landPassMap[i];
@@ -175,10 +174,18 @@ m.NavalManager.prototype.init = function(gameState, deserializing)
 		this.setDockIndex(gameState, dock);
 };
 
-m.NavalManager.prototype.resetFishingBoats = function(gameState)
+m.NavalManager.prototype.updateFishingBoats = function(sea, num)
 {
-	for (let i = 0; i < gameState.ai.accessibility.regionSize.length; ++i)
-		this.wantedFishShips[i] = 0;
+	if (this.wantedFishShips[sea])
+		this.wantedFishShips[sea] = num;
+};
+
+m.NavalManager.prototype.resetFishingBoats = function(gameState, sea)
+{
+	if (sea !== undefined)
+		this.wantedFishShips[sea] = 0;
+	else
+		this.wantedFishShips.fill(0);
 };
 
 m.NavalManager.prototype.setShipIndex = function(gameState, ship)
@@ -189,13 +196,13 @@ m.NavalManager.prototype.setShipIndex = function(gameState, ship)
 
 m.NavalManager.prototype.setDockIndex = function(gameState, dock)
 {
-	var land = dock.getMetadata(PlayerID, "access");
+	let land = dock.getMetadata(PlayerID, "access");
 	if (land === undefined)
 	{
 		land = this.getDockIndex(gameState, dock, false);
 		dock.setMetadata(PlayerID, "access", land);
 	}
-	var sea = dock.getMetadata(PlayerID, "sea");
+	let sea = dock.getMetadata(PlayerID, "sea");
 	if (sea === undefined)
 	{
 		sea = this.getDockIndex(gameState, dock, true);
@@ -203,16 +210,18 @@ m.NavalManager.prototype.setDockIndex = function(gameState, dock)
 	}
 };
 
-// get the indices for our starting docks and those of our allies
-// land index when onWater=false, sea indes when true
+/**
+ * get the indices for our starting docks and those of our allies
+ * land index when onWater=false, sea indes when true
+ */
 m.NavalManager.prototype.getDockIndex = function(gameState, dock, onWater)
 {
-	var index = gameState.ai.accessibility.getAccessValue(dock.position(), onWater);
+	let index = gameState.ai.accessibility.getAccessValue(dock.position(), onWater);
 	if (index < 2)
 	{
 		// pre-positioned docks are sometimes not well positionned
-		var dockPos = dock.position();
-		var radius = dock.footprintRadius();
+		let dockPos = dock.position();
+		let radius = dock.footprintRadius();
 		for (let i = 0; i < 16; i++)
 		{
 			let pos = [ dockPos[0] + radius*Math.cos(i*Math.PI/8), dockPos[1] + radius*Math.sin(i*Math.PI/8)];
@@ -227,11 +236,11 @@ m.NavalManager.prototype.getDockIndex = function(gameState, dock, onWater)
 	return index;
 };
 
-// get the list of seas around this region not connected by a dock
+/** get the list of seas (or lands) around this region not connected by a dock */
 m.NavalManager.prototype.getUnconnectedSeas = function(gameState, region)
 {
-	var seas = gameState.ai.accessibility.regionLinks[region].slice();
-	var docks = gameState.getOwnStructures().filter(API3.Filters.byClass("Dock"));
+	let seas = gameState.ai.accessibility.regionLinks[region].slice();
+	let docks = gameState.getOwnStructures().filter(API3.Filters.byClass("Dock"));
 	docks.forEach(function (dock) {
 		if (dock.getMetadata(PlayerID, "access") !== region)
 			return;
@@ -244,7 +253,7 @@ m.NavalManager.prototype.getUnconnectedSeas = function(gameState, region)
 
 m.NavalManager.prototype.checkEvents = function(gameState, queues, events)
 {
-	for (let evt of events["ConstructionFinished"])
+	for (let evt of events.ConstructionFinished)
 	{
 		if (!evt || !evt.newentity)
 			continue;
@@ -253,7 +262,7 @@ m.NavalManager.prototype.checkEvents = function(gameState, queues, events)
 			this.setDockIndex(gameState, entity);
 	}
 
-	for (let evt of events["TrainingFinished"])
+	for (let evt of events.TrainingFinished)
 	{
 		if (!evt || !evt.entities)
 			continue;
@@ -266,33 +275,33 @@ m.NavalManager.prototype.checkEvents = function(gameState, queues, events)
 		}
 	}
 
-	for (let evt of events["Destroy"])
+	for (let evt of events.Destroy)
 	{
 		if (!evt.entityObj || evt.entityObj.owner() !== PlayerID || !evt.metadata || !evt.metadata[PlayerID])
 			continue;
-		if (!evt.entityObj.hasClass("Ship") || !evt.metadata[PlayerID]["transporter"])
+		if (!evt.entityObj.hasClass("Ship") || !evt.metadata[PlayerID].transporter)
 			continue;
-		var plan = this.getPlan(evt.metadata[PlayerID]["transporter"]);
+		let plan = this.getPlan(evt.metadata[PlayerID].transporter);
 		if (!plan)
 			continue;
 
-		var shipId = evt.entityObj.id();
+		let shipId = evt.entityObj.id();
 		if (this.Config.debug > 1)
 			API3.warn("one ship " + shipId + " from plan " + plan.ID + " destroyed during " + plan.state);
 		if (plan.state === "boarding")
 		{
 			// just reset the units onBoard metadata and wait for a new ship to be assigned to this plan
 			plan.units.forEach(function (ent) {
-				if ((ent.getMetadata(PlayerID, "onBoard") === "onBoard" && ent.position())
-					|| ent.getMetadata(PlayerID, "onBoard") === shipId)
+				if ((ent.getMetadata(PlayerID, "onBoard") === "onBoard" && ent.position()) ||
+					ent.getMetadata(PlayerID, "onBoard") === shipId)
 					ent.setMetadata(PlayerID, "onBoard", undefined);
 			});
-			plan.needTransportShips = (plan.transportShips.length === 0);
+			plan.needTransportShips = !plan.transportShips.hasEntities();
 		}
 		else if (plan.state === "sailing")
 		{
-			var endIndex = plan.endIndex;
-			var self = this;
+			let endIndex = plan.endIndex;
+			let self = this;
 			plan.units.forEach(function (ent) {
 				if (!ent.position())  // unit from another ship of this plan ... do nothing
 					return;
@@ -311,7 +320,7 @@ m.NavalManager.prototype.checkEvents = function(gameState, queues, events)
 		}
 	}
 
-	for (let evt of events["OwnershipChanged"])	// capture events
+	for (let evt of events.OwnershipChanged)	// capture events
 	{
 		if (evt.to === PlayerID)
 		{
@@ -336,15 +345,17 @@ m.NavalManager.prototype.addPlan = function(plan)
 	this.transportPlans.push(plan);
 };
 
-// complete already existing plan or create a new one for this requirement
-// (many units can then call this separately and end up in the same plan)
-// TODO  check garrison classes
+/**
+ * complete already existing plan or create a new one for this requirement
+ * (many units can then call this separately and end up in the same plan)
+ * TODO  check garrison classes
+ */
 m.NavalManager.prototype.requireTransport = function(gameState, entity, startIndex, endIndex, endPos)
 {
 	if (entity.getMetadata(PlayerID, "transport") !== undefined)
 	{
 		if (this.Config.debug > 0)
-			API3.warn("Petra naval manager error: unit " + entity.id() +  " has already required a transport");
+			API3.warn("Petra naval manager error: unit " + entity.id() + " has already required a transport");
 		return false;
 	}
 
@@ -369,12 +380,12 @@ m.NavalManager.prototype.requireTransport = function(gameState, entity, startInd
 	return true;
 };
 
-// split a transport plan in two, moving all entities not yet affected to a ship in the new plan
+/** split a transport plan in two, moving all entities not yet affected to a ship in the new plan */
 m.NavalManager.prototype.splitTransport = function(gameState, plan)
 {
 	if (this.Config.debug > 1)
 		API3.warn(">>>> split of transport plan started <<<<");
-	var newplan = new m.TransportPlan(gameState, [], plan.startIndex, plan.endIndex, plan.endPos);
+	let newplan = new m.TransportPlan(gameState, [], plan.startIndex, plan.endIndex, plan.endPos);
 	if (newplan.failed)
 	{
 		if (this.Config.debug > 1)
@@ -383,7 +394,7 @@ m.NavalManager.prototype.splitTransport = function(gameState, plan)
 	}
 	newplan.init(gameState);
 
-	var nbUnits = 0;
+	let nbUnits = 0;
 	plan.units.forEach(function (ent) {
 		if (ent.getMetadata(PlayerID, "onBoard"))
 			return;
@@ -394,7 +405,7 @@ m.NavalManager.prototype.splitTransport = function(gameState, plan)
 		API3.warn(">>>> previous plan left with units " + plan.units.length);
 	if (nbUnits)
 		this.transportPlans.push(newplan);
-	return (nbUnits !== 0);
+	return nbUnits !== 0;
 };
 
 /**
@@ -439,7 +450,7 @@ m.NavalManager.prototype.setMinimalTransportShips = function(gameState, sea, num
 // bumps up the number of ships we want if we need more.
 m.NavalManager.prototype.checkLevels = function(gameState, queues)
 {
-	if (queues.ships.length() !== 0)
+	if (queues.ships.hasQueuedUnits())
 		return;
 
 	for (let sea = 0; sea < this.neededTransportShips.length; sea++)
@@ -450,8 +461,8 @@ m.NavalManager.prototype.checkLevels = function(gameState, queues)
 		if (!plan.needTransportShips || plan.units.length < 2)
 			continue;
 		let sea = plan.sea;
-		if (gameState.countOwnQueuedEntitiesWithMetadata("sea", sea) > 0
-			|| this.seaTransportShips[sea].length < this.wantedTransportShips[sea])
+		if (gameState.countOwnQueuedEntitiesWithMetadata("sea", sea) > 0 ||
+			this.seaTransportShips[sea].length < this.wantedTransportShips[sea])
 			continue;
 		++this.neededTransportShips[sea];
 		if (this.wantedTransportShips[sea] === 0 || this.seaTransportShips[sea].length < plan.transportShips.length + 2)
@@ -468,13 +479,13 @@ m.NavalManager.prototype.checkLevels = function(gameState, queues)
 
 m.NavalManager.prototype.maintainFleet = function(gameState, queues)
 {
-	if (queues.ships.length() > 0)
+	if (queues.ships.hasQueuedUnits())
 		return;
-	if (gameState.getOwnEntitiesByClass("Dock", true).filter(API3.Filters.isBuilt()).length +
-		gameState.getOwnEntitiesByClass("Shipyard", true).filter(API3.Filters.isBuilt()).length === 0)
+	if (!gameState.getOwnEntitiesByClass("Dock", true).filter(API3.Filters.isBuilt()).hasEntities() &&
+	    !gameState.getOwnEntitiesByClass("Shipyard", true).filter(API3.Filters.isBuilt()).hasEntities())
 		return;
 	// check if we have enough transport ships per region.
-	for (var sea = 0; sea < this.seaShips.length; ++sea)
+	for (let sea = 0; sea < this.seaShips.length; ++sea)
 	{
 		if (this.seaShips[sea] === undefined)
 			continue;
@@ -483,10 +494,10 @@ m.NavalManager.prototype.maintainFleet = function(gameState, queues)
 
 		if (this.seaTransportShips[sea].length < this.wantedTransportShips[sea])
 		{
-			var template = this.getBestShip(gameState, sea, "transport");
+			let template = this.getBestShip(gameState, sea, "transport");
 			if (template)
 			{
-				queues.ships.addItem(new m.TrainingPlan(gameState, template, { "sea": sea }, 1, 1));
+				queues.ships.addPlan(new m.TrainingPlan(gameState, template, { "sea": sea }, 1, 1));
 				continue;
 			}
 		}
@@ -497,14 +508,14 @@ m.NavalManager.prototype.maintainFleet = function(gameState, queues)
 			let template = this.getBestShip(gameState, sea, "fishing");
 			if (template)
 			{
-				queues.ships.addItem(new m.TrainingPlan(gameState, template, { "base": 0, "role": "worker", "sea": sea }, 1, 1));
+				queues.ships.addPlan(new m.TrainingPlan(gameState, template, { "base": 0, "role": "worker", "sea": sea }, 1, 1));
 				continue;
 			}
 		}
 	}
 };
 
-// assigns free ships to plans that need some
+/** assigns free ships to plans that need some */
 m.NavalManager.prototype.assignShipsToPlans = function(gameState)
 {
 	for (let plan of this.transportPlans)
@@ -512,15 +523,15 @@ m.NavalManager.prototype.assignShipsToPlans = function(gameState)
 			plan.assignShip(gameState);
 };
 
-// let blocking ships move apart from active ships (waiting for a better pathfinder)
+/** let blocking ships move apart from active ships (waiting for a better pathfinder) */
 m.NavalManager.prototype.moveApart = function(gameState)
 {
-	var self = this;
+	let self = this;
 
 	this.ships.forEach(function(ship) {
 		if (ship.hasClass("FishingBoat"))   // small ships should not be a problem
 			return;
-		var sea = ship.getMetadata(PlayerID, "sea");
+		let sea = ship.getMetadata(PlayerID, "sea");
 		if (ship.getMetadata(PlayerID, "transporter") === undefined)
 		{
 			if (ship.isIdle())	// do not stay idle near a dock to not disturb other ships
@@ -551,7 +562,7 @@ m.NavalManager.prototype.moveApart = function(gameState)
 	gameState.ai.HQ.tradeManager.traders.filter(API3.Filters.byClass("Ship")).forEach(function(ship) {
 		if (ship.getMetadata(PlayerID, "route") === undefined)
 			return;
-		var sea = ship.getMetadata(PlayerID, "sea");
+		let sea = ship.getMetadata(PlayerID, "sea");
 		self.seaShips[sea].forEach(function(blockingShip) {
 			if (blockingShip === ship || !blockingShip.isIdle())
 				return;
@@ -572,23 +583,25 @@ m.NavalManager.prototype.buildNavalStructures = function(gameState, queues)
 
 	if (gameState.getPopulation() > this.Config.Economy.popForDock)
 	{
-		if (queues.dock.countQueuedUnitsWithClass("NavalMarket") === 0
-			&& gameState.getOwnStructures().filter(API3.Filters.and(API3.Filters.byClass("NavalMarket"), API3.Filters.isFoundation())).length === 0
-			&& gameState.ai.HQ.canBuild(gameState, "structures/{civ}_dock"))
+		if (queues.dock.countQueuedUnitsWithClass("NavalMarket") === 0 &&
+			!gameState.getOwnStructures().filter(API3.Filters.and(API3.Filters.byClass("NavalMarket"), API3.Filters.isFoundation())).hasEntities() &&
+			gameState.ai.HQ.canBuild(gameState, "structures/{civ}_dock"))
 		{
-			var dockStarted = false;
-			for (var base of gameState.ai.HQ.baseManagers)
+			let dockStarted = false;
+			for (let base of gameState.ai.HQ.baseManagers)
 			{
 				if (dockStarted)
 					break;
 				if (!base.anchor || base.constructing)
 					continue;
-				var remaining = this.getUnconnectedSeas(gameState, base.accessIndex);
-				for (var sea of remaining)
+				let remaining = this.getUnconnectedSeas(gameState, base.accessIndex);
+				for (let sea of remaining)
 				{
 					if (!gameState.ai.HQ.navalRegions[sea])
 						continue;
-					queues.dock.addItem(new m.ConstructionPlan(gameState, "structures/{civ}_dock", { "land": [base.accessIndex], "sea": sea }));
+					let wantedLand = {};
+					wantedLand[base.accessIndex] = true;
+					queues.dock.addPlan(new m.ConstructionPlan(gameState, "structures/{civ}_dock", { "land": wantedLand, "sea": sea }));
 					dockStarted = true;
 					break;
 				}
@@ -596,14 +609,14 @@ m.NavalManager.prototype.buildNavalStructures = function(gameState, queues)
 		}
 	}
 
-	if (gameState.currentPhase() < 2 || gameState.getPopulation() < this.Config.Economy.popForTown + 15
-		|| queues.militaryBuilding.length() !== 0 || this.bNaval.length === 0)
+	if (gameState.currentPhase() < 2 || gameState.getPopulation() < this.Config.Economy.popForTown + 15 ||
+		queues.militaryBuilding.hasQueuedUnits() || this.bNaval.length === 0)
 		return;
-	var docks = gameState.getOwnStructures().filter(API3.Filters.byClass("Dock"));
-	if (!docks.length)
+	let docks = gameState.getOwnStructures().filter(API3.Filters.byClass("Dock"));
+	if (!docks.hasEntities())
 		return;
-	var nNaval = 0;
-	for (var naval of this.bNaval)
+	let nNaval = 0;
+	for (let naval of this.bNaval)
 		nNaval += gameState.countEntitiesAndQueuedByType(naval, true);
 
 	if (nNaval === 0 || (nNaval < this.bNaval.length && gameState.getPopulation() > 120))
@@ -612,32 +625,28 @@ m.NavalManager.prototype.buildNavalStructures = function(gameState, queues)
 		{
 			if (gameState.countEntitiesAndQueuedByType(naval, true) < 1 && gameState.ai.HQ.canBuild(gameState, naval))
 			{
-				let land = [];
+				let wantedLand = {};
 				for (let base of gameState.ai.HQ.baseManagers)
-				{
-					if (!base.anchor)
-						continue;
-					if (land.indexOf(base.accessIndex) === -1)
-						land.push(base.accessIndex);
-				}
+					if (base.anchor)
+						wantedLand[base.accessIndex] = true;
 				let sea = docks.toEntityArray()[0].getMetadata(PlayerID, "sea");
-				queues.militaryBuilding.addItem(new m.ConstructionPlan(gameState, naval, { "land": land, "sea": sea }));
+				queues.militaryBuilding.addPlan(new m.ConstructionPlan(gameState, naval, { "land": wantedLand, "sea": sea }));
 				break;
 			}
 		}
 	}
 };
 
-// goal can be either attack (choose ship with best arrowCount) or transport (choose ship with best capacity)
+/** goal can be either attack (choose ship with best arrowCount) or transport (choose ship with best capacity) */
 m.NavalManager.prototype.getBestShip = function(gameState, sea, goal)
 {
-	var civ = gameState.civ();
-	var trainableShips = [];
+	let civ = gameState.civ();
+	let trainableShips = [];
 	gameState.getOwnTrainingFacilities().filter(API3.Filters.byMetadata(PlayerID, "sea", sea)).forEach(function(ent) {
-		var trainables = ent.trainableEntities(civ);
+		let trainables = ent.trainableEntities(civ);
 		for (let trainable of trainables)
 		{
-			if (gameState.isDisabledTemplates(trainable))
+			if (gameState.isTemplateDisabled(trainable))
 				continue;
 			let template = gameState.getTemplate(trainable);
 			if (template && template.hasClass("Ship") && trainableShips.indexOf(trainable) === -1)
@@ -645,10 +654,10 @@ m.NavalManager.prototype.getBestShip = function(gameState, sea, goal)
 		}
 	});
 
-	var best = 0;
-	var bestShip;
-	var limits = gameState.getEntityLimits();
-	var current = gameState.getEntityCounts();
+	let best = 0;
+	let bestShip;
+	let limits = gameState.getEntityLimits();
+	let current = gameState.getEntityCounts();
 	for (let trainable of trainableShips)
 	{
 		let template = gameState.getTemplate(trainable);
