@@ -22,7 +22,7 @@ m.HQ.prototype.gameAnalysis = function(gameState)
 	nobase.init(gameState);
 	nobase.accessIndex = 0;
 	this.baseManagers.push(nobase);   // baseManagers[0] will deal with unit/structure without base
-	var ccEnts = gameState.getOwnStructures().filter(API3.Filters.byClass("CivCentre"));
+	let ccEnts = gameState.getOwnStructures().filter(API3.Filters.byClass("CivCentre"));
 	for (let cc of ccEnts.values())
 	{
 		let newbase = new m.BaseManager(gameState, this.Config);
@@ -37,10 +37,10 @@ m.HQ.prototype.gameAnalysis = function(gameState)
 
 	// If no base yet, check if we can construct one. If not, dispatch our units to possible tasks/attacks
 	this.canBuildUnits = true;
-	if (!gameState.getOwnStructures().filter(API3.Filters.byClass("CivCentre")).length)
+	if (!gameState.getOwnStructures().filter(API3.Filters.byClass("CivCentre")).hasEntities())
 	{
-		var template = gameState.applyCiv("structures/{civ}_civil_centre");
-		if (gameState.isDisabledTemplates(template) || !gameState.getTemplate(template).available(gameState))
+		let template = gameState.applyCiv("structures/{civ}_civil_centre");
+		if (gameState.isTemplateDisabled(template) || !gameState.getTemplate(template).available(gameState))
 		{
 			if (this.Config.debug > 1)
 				API3.warn(" this AI is unable to produce any units");
@@ -61,13 +61,13 @@ m.HQ.prototype.gameAnalysis = function(gameState)
  */
 m.HQ.prototype.assignStartingEntities = function(gameState)
 {
-	for (var ent of gameState.getOwnEntities().values())
+	for (let ent of gameState.getOwnEntities().values())
 	{
 		// do not affect merchant ship immediately to trade as they may-be useful for transport
 		if (ent.hasClass("Trader") && !ent.hasClass("Ship"))
 			this.tradeManager.assignTrader(ent);
 
-		var pos = ent.position();
+		let pos = ent.position();
 		if (!pos)
 		{
 			// TODO should support recursive garrisoning. Make a warning for now
@@ -94,8 +94,8 @@ m.HQ.prototype.assignStartingEntities = function(gameState)
 
 		ent.setMetadata(PlayerID, "access", gameState.ai.accessibility.getAccessValue(pos));
 		let bestbase;
-		var territorypos = this.territoryMap.gamePosToMapPos(pos);
-		var territoryIndex = territorypos[0] + territorypos[1]*this.territoryMap.width;
+		let territorypos = this.territoryMap.gamePosToMapPos(pos);
+		let territoryIndex = territorypos[0] + territorypos[1]*this.territoryMap.width;
 		for (let i = 1; i < this.baseManagers.length; ++i)
 		{
 			let base = this.baseManagers[i];
@@ -133,10 +133,10 @@ m.HQ.prototype.assignStartingEntities = function(gameState)
  */
 m.HQ.prototype.regionAnalysis = function(gameState)
 {
-	var accessibility = gameState.ai.accessibility;
+	let accessibility = gameState.ai.accessibility;
 	let landIndex;
 	let seaIndex;
-	var ccEnts = gameState.getOwnStructures().filter(API3.Filters.byClass("CivCentre"));
+	let ccEnts = gameState.getOwnStructures().filter(API3.Filters.byClass("CivCentre"));
 	for (let cc of ccEnts.values())
 	{
 		let land = accessibility.getAccessValue(cc.position());
@@ -148,7 +148,7 @@ m.HQ.prototype.regionAnalysis = function(gameState)
 	}
 	if (!landIndex)
 	{
-		var civ = gameState.civ();
+		let civ = gameState.civ();
 		for (let ent of gameState.getOwnEntities().values())
 		{
 			if (!ent.position() || (!ent.hasClass("Unit") && !ent.trainableEntities(civ)))
@@ -170,12 +170,12 @@ m.HQ.prototype.regionAnalysis = function(gameState)
 		return false;
 	}
 
-	var passabilityMap = gameState.getMap();
-	var totalSize = passabilityMap.width * passabilityMap.width;
-	var minLandSize = Math.floor(0.1*totalSize);
-	var minWaterSize = Math.floor(0.2*totalSize);
-	var cellArea = passabilityMap.cellSize * passabilityMap.cellSize;  
-	for (var i = 0; i < accessibility.regionSize.length; ++i)
+	let passabilityMap = gameState.getMap();
+	let totalSize = passabilityMap.width * passabilityMap.width;
+	let minLandSize = Math.floor(0.1*totalSize);
+	let minWaterSize = Math.floor(0.2*totalSize);
+	let cellArea = passabilityMap.cellSize * passabilityMap.cellSize;  
+	for (let i = 0; i < accessibility.regionSize.length; ++i)
 	{
 		if (landIndex && i == landIndex)
 			this.landRegions[i] = true;
@@ -183,7 +183,7 @@ m.HQ.prototype.regionAnalysis = function(gameState)
 		{
 			if (landIndex)
 			{
-				var sea = this.getSeaIndex(gameState, landIndex, i);
+				let sea = this.getSeaIndex(gameState, landIndex, i);
 				if (sea && (accessibility.regionSize[i] > minLandSize || accessibility.regionSize[sea] > minWaterSize))
 				{
 					this.navalMap = true;
@@ -193,7 +193,7 @@ m.HQ.prototype.regionAnalysis = function(gameState)
 			}
 			else
 			{
-				var traject = accessibility.getTrajectToIndex(seaIndex, i);
+				let traject = accessibility.getTrajectToIndex(seaIndex, i);
 				if (traject && traject.length === 2)
 				{
 					this.navalMap = true;
@@ -213,7 +213,7 @@ m.HQ.prototype.regionAnalysis = function(gameState)
 
 	if (this.Config.debug < 3)
 		return true;
-	for (var region in this.landRegions)
+	for (let region in this.landRegions)
 		API3.warn(" >>> zone " + region + " taille " + cellArea*gameState.ai.accessibility.regionSize[region]);
 	API3.warn(" navalMap " + this.navalMap);
 	API3.warn(" landRegions " + uneval(this.landRegions));
@@ -227,20 +227,17 @@ m.HQ.prototype.regionAnalysis = function(gameState)
  */
 m.HQ.prototype.structureAnalysis = function(gameState)
 {
-	var civ = gameState.playerData.civ;
-	if (civ in this.Config.buildings.base)
-		this.bBase = this.Config.buildings.base[civ];
-	else
-		this.bBase = this.Config.buildings.base['default'];
+	let civref = gameState.playerData.civ;
+	let civ = civref in this.Config.buildings.base ? civref : 'default';
+	this.bBase = [];
+	for (let base of this.Config.buildings.base[civ])
+		this.bBase.push(gameState.applyCiv(base));
 
-	if (civ in this.Config.buildings.advanced)
-		this.bAdvanced = this.Config.buildings.advanced[civ];
-	else
-		this.bAdvanced = this.Config.buildings.advanced['default'];	
-	for (var i in this.bBase)
-		this.bBase[i] = gameState.applyCiv(this.bBase[i]);
-	for (var i in this.bAdvanced)
-		this.bAdvanced[i] = gameState.applyCiv(this.bAdvanced[i]);
+	civ = civref in this.Config.buildings.advanced ? civref : 'default';
+	this.bAdvanced = [];
+	for (let advanced of this.Config.buildings.advanced[civ])
+		if (!gameState.isTemplateDisabled(gameState.applyCiv(advanced)))
+			this.bAdvanced.push(gameState.applyCiv(advanced));
 };
 
 /**
@@ -249,25 +246,25 @@ m.HQ.prototype.structureAnalysis = function(gameState)
  */
 m.HQ.prototype.buildFirstBase = function(gameState)
 {
-	var total = gameState.getResources();
-	var template = gameState.applyCiv("structures/{civ}_civil_centre");
-	if (gameState.isDisabledTemplates(template))
+	let total = gameState.getResources();
+	let template = gameState.applyCiv("structures/{civ}_civil_centre");
+	if (gameState.isTemplateDisabled(template))
 		return;
-	var template = gameState.getTemplate(template);
-	var goal = "civil_centre";
-	var docks = gameState.getOwnStructures().filter(API3.Filters.byClass("Dock"));
-	if (!total.canAfford(new API3.Resources(template.cost())) && !docks.length)
+	template = gameState.getTemplate(template);
+	let goal = "civil_centre";
+	let docks = gameState.getOwnStructures().filter(API3.Filters.byClass("Dock"));
+	if (!total.canAfford(new API3.Resources(template.cost())) && !docks.hasEntities())
 	{
 		// not enough resource to build a cc, try with a dock to accumulate resources if none yet
-		if (gameState.ai.queues.dock.countQueuedUnits())
+		if (gameState.ai.queues.dock.hasQueuedUnits())
 			return;
 		template = gameState.applyCiv("structures/{civ}_dock");
-		if (gameState.isDisabledTemplates(template))
+		if (gameState.isTemplateDisabled(template))
 			return;
 		template = gameState.getTemplate(template);
 		if (!total.canAfford(new API3.Resources(template.cost())))
 			return;
-		var goal = "dock";
+		goal = "dock";
 	}
 
 	// We first choose as startingPoint the point where we have the more units  
@@ -315,10 +312,10 @@ m.HQ.prototype.buildFirstBase = function(gameState)
 	if (goal === "dock")
 	{
 		let sea = startingPoint[imax].sea > 1 ? startingPoint[imax].sea : undefined;
-		gameState.ai.queues.dock.addItem(new m.ConstructionPlan(gameState, "structures/{civ}_dock", { "sea": sea, "proximity": startingPoint[imax].pos }));
+		gameState.ai.queues.dock.addPlan(new m.ConstructionPlan(gameState, "structures/{civ}_dock", { "sea": sea, "proximity": startingPoint[imax].pos }));
 	}
 	else
-		gameState.ai.queues.civilCentre.addItem(new m.ConstructionPlan(gameState, "structures/{civ}_civil_centre", { "base": -1, "resource": "wood", "proximity": startingPoint[imax].pos }));
+		gameState.ai.queues.civilCentre.addPlan(new m.ConstructionPlan(gameState, "structures/{civ}_civil_centre", { "base": -1, "resource": "wood", "proximity": startingPoint[imax].pos }));
 };
 
 /**
@@ -328,23 +325,23 @@ m.HQ.prototype.buildFirstBase = function(gameState)
  */
 m.HQ.prototype.dispatchUnits = function(gameState)
 {
-	var allycc = gameState.getExclusiveAllyEntities().filter(API3.Filters.byClass("CivCentre")).toEntityArray();
+	let allycc = gameState.getExclusiveAllyEntities().filter(API3.Filters.byClass("CivCentre")).toEntityArray();
 	if (allycc.length)
 	{
 		if (this.Config.debug > 1)
 			API3.warn(" We have allied cc " + allycc.length + " and " + gameState.getOwnUnits().length + " units ");
-		var units = gameState.getOwnUnits();
-		var num = Math.max(Math.min(Math.round(0.08*(1+this.Config.personality.cooperative)*units.length), 20), 5);
-		var num1 = Math.floor(num / 2);
-		var num2 = num1;
+		let units = gameState.getOwnUnits();
+		let num = Math.max(Math.min(Math.round(0.08*(1+this.Config.personality.cooperative)*units.length), 20), 5);
+		let num1 = Math.floor(num / 2);
+		let num2 = num1;
 		// first pass to affect ranged infantry
 		units.filter(API3.Filters.byClassesAnd(["Infantry", "Ranged"])).forEach(function (ent) {
 			if (!num || !num1)
 				return;
 			if (ent.getMetadata(PlayerID, "allied"))
 				return;
-			var access = gameState.ai.accessibility.getAccessValue(ent.position());
-			for (var cc of allycc)
+			let access = gameState.ai.accessibility.getAccessValue(ent.position());
+			for (let cc of allycc)
 			{
 				if (!cc.position())
 					continue;
@@ -353,7 +350,7 @@ m.HQ.prototype.dispatchUnits = function(gameState)
 				--num;
 				--num1;
 				ent.setMetadata(PlayerID, "allied", true);
-				var range = 1.5 * cc.footprintRadius();
+				let range = 1.5 * cc.footprintRadius();
 				ent.moveToRange(cc.position()[0], cc.position()[1], range, range);
 				break;
 			}
@@ -364,8 +361,8 @@ m.HQ.prototype.dispatchUnits = function(gameState)
 				return;
 			if (ent.getMetadata(PlayerID, "allied"))
 				return;
-			var access = gameState.ai.accessibility.getAccessValue(ent.position());
-			for (var cc of allycc)
+			let access = gameState.ai.accessibility.getAccessValue(ent.position());
+			for (let cc of allycc)
 			{
 				if (!cc.position())
 					continue;
@@ -374,7 +371,7 @@ m.HQ.prototype.dispatchUnits = function(gameState)
 				--num;
 				--num2;
 				ent.setMetadata(PlayerID, "allied", true);
-				var range = 1.5 * cc.footprintRadius();
+				let range = 1.5 * cc.footprintRadius();
 				ent.moveToRange(cc.position()[0], cc.position()[1], range, range);
 				break;
 			}
@@ -385,8 +382,8 @@ m.HQ.prototype.dispatchUnits = function(gameState)
 				return;
 			if (ent.getMetadata(PlayerID, "allied"))
 				return;
-			var access = gameState.ai.accessibility.getAccessValue(ent.position());
-			for (var cc of allycc)
+			let access = gameState.ai.accessibility.getAccessValue(ent.position());
+			for (let cc of allycc)
 			{
 				if (!cc.position())
 					continue;
@@ -395,7 +392,7 @@ m.HQ.prototype.dispatchUnits = function(gameState)
 				if (!ent.hasClass("Support"))
 					--num;
 				ent.setMetadata(PlayerID, "allied", true);
-				var range = 1.5 * cc.footprintRadius();
+				let range = 1.5 * cc.footprintRadius();
 				ent.moveToRange(cc.position()[0], cc.position()[1], range, range);
 				break;
 			}
@@ -413,7 +410,8 @@ m.HQ.prototype.configFirstBase = function(gameState)
 	if (this.baseManagers.length < 2)
 		return;
 
-	var startingSize = 0;
+	let startingSize = 0;
+	let startingLand = [];
 	for (let region in this.landRegions)
 	{
 		for (let base of this.baseManagers)
@@ -421,10 +419,11 @@ m.HQ.prototype.configFirstBase = function(gameState)
 			if (!base.anchor || base.accessIndex != +region)
 				continue;
 			startingSize += gameState.ai.accessibility.regionSize[region];
+			startingLand.push(base.accessIndex);
 			break;
 		}
 	}
-	var cell = gameState.getMap().cellSize;
+	let cell = gameState.getMap().cellSize;
 	startingSize = startingSize * cell * cell;
 	if (this.Config.debug > 1)
 		API3.warn("starting size " + startingSize + "(cut at 24000 for fish pushing)");
@@ -432,17 +431,49 @@ m.HQ.prototype.configFirstBase = function(gameState)
 	{
 		this.saveSpace = true;
 		this.Config.Economy.popForDock = Math.min(this.Config.Economy.popForDock, 16);
-		this.Config.Economy.targetNumFishers = Math.max(this.Config.Economy.targetNumFishers, 2);
+		let num = Math.max(this.Config.Economy.targetNumFishers, 2);
+		for (let land of startingLand)
+		{
+			for (let sea of gameState.ai.accessibility.regionLinks[land])
+				if (gameState.ai.HQ.navalRegions[sea])
+					this.navalManager.updateFishingBoats(sea, num);
+		}
 	}
 
-	// - count the available wood resource, and allow rushes only if enough (we should otherwise favor expansion)
-	var startingWood = gameState.getResources()["wood"];
-	var check = {};
-	for (var proxim of ["nearby", "medium", "faraway"])
+	// - count the available wood resource, and react accordingly
+	let startingFood = gameState.getResources().food;
+	let check = {};
+	for (let proxim of ["nearby", "medium", "faraway"])
 	{
 		for (let base of this.baseManagers)
 		{
-			for (var supply of base.dropsiteSupplies["wood"][proxim])
+			for (let supply of base.dropsiteSupplies.food[proxim])
+			{
+				if (check[supply.id])    // avoid double counting as same resource can appear several time
+					continue;
+				check[supply.id] = true;
+				startingFood += supply.ent.resourceSupplyAmount();
+			}
+		}
+	}
+	if (startingFood < 800)
+	{
+		if (startingSize < 24000)
+		{
+			this.needFish = true;
+			this.Config.Economy.popForDock = 1;
+		}
+		else
+			this.needFarm = true;
+	}
+	// - count the available wood resource, and allow rushes only if enough (we should otherwise favor expansion)
+	let startingWood = gameState.getResources().wood;
+	check = {};
+	for (let proxim of ["nearby", "medium", "faraway"])
+	{
+		for (let base of this.baseManagers)
+		{
+			for (let supply of base.dropsiteSupplies.wood[proxim])
 			{
 				if (check[supply.id])    // avoid double counting as same resource can appear several time
 					continue;
@@ -454,7 +485,15 @@ m.HQ.prototype.configFirstBase = function(gameState)
 	if (this.Config.debug > 1)
 		API3.warn("startingWood: " + startingWood + " (cut at 8500 for no rush and 6000 for saveResources)");
 	if (startingWood < 6000)
+	{
 		this.saveResources = true;
+		this.Config.Economy.popForTown = 40; // Switch to town phase as soon as possible to be able to expand
+		if (startingWood < 2000 && this.needFarm)
+		{
+			this.needCorral = true;
+			this.needFarm = false;
+		}
+	}
 	if (startingWood > 8500 && this.canBuildUnits)
 	{
 		let allowed = Math.ceil((startingWood - 8500) / 3000);
@@ -462,8 +501,8 @@ m.HQ.prototype.configFirstBase = function(gameState)
 	}
 
 	// immediatly build a wood dropsite if possible.
-	var template = gameState.applyCiv("structures/{civ}_storehouse");
-	if (gameState.getOwnEntitiesByClass("Storehouse", true).length === 0 && this.canBuild(gameState, template))
+	let template = gameState.applyCiv("structures/{civ}_storehouse");
+	if (!gameState.getOwnEntitiesByClass("Storehouse", true).hasEntities() && this.canBuild(gameState, template))
 	{
 		let newDP = this.baseManagers[1].findBestDropsiteLocation(gameState, "wood");
 		if (newDP.quality > 40)
@@ -471,15 +510,22 @@ m.HQ.prototype.configFirstBase = function(gameState)
 			// if we start with enough workers, put our available resources in this first dropsite
 			// same thing if our pop exceed the allowed one, as we will need several houses
 			let numWorkers = gameState.getOwnUnits().filter(API3.Filters.byClass("Worker")).length;
-			if ((numWorkers > 12 && newDP.quality > 60)
-				|| (gameState.getPopulation() > gameState.getPopulationLimit() + 20))
+			if ((numWorkers > 12 && newDP.quality > 60) ||
+				(gameState.getPopulation() > gameState.getPopulationLimit() + 20))
 			{
 				let cost = new API3.Resources(gameState.getTemplate(template).cost());
 				gameState.ai.queueManager.setAccounts(gameState, cost, "dropsites");
 			}
-			gameState.ai.queues.dropsites.addItem(new m.ConstructionPlan(gameState, template, { "base": this.baseManagers[1].ID }, newDP.pos));
+			gameState.ai.queues.dropsites.addPlan(new m.ConstructionPlan(gameState, template, { "base": this.baseManagers[1].ID }, newDP.pos));
 		}
 	}
+	// and build immediately a corral if needed
+	if (this.needCorral)
+	{
+		template = gameState.applyCiv("structures/{civ}_corral");
+		if (!gameState.getOwnEntitiesByClass("Corral", true).hasEntities() && this.canBuild(gameState, template))
+			gameState.ai.queues.corral.addPlan(new m.ConstructionPlan(gameState, template, { "base": this.baseManagers[1].ID }));
+	}
 };
 
 return m;
