@@ -48,7 +48,7 @@ GuiInterface.prototype.Init = function()
  * Returns global information about the current game state.
  * This is used by the GUI and also by AI scripts.
  */
-GuiInterface.prototype.GetSimulationState = function(player)
+GuiInterface.prototype.GetSimulationState = function()
 {
 	let ret = {
 		"players": []
@@ -107,6 +107,7 @@ GuiInterface.prototype.GetSimulationState = function(player)
 			"cheatsEnabled": cmpPlayer.GetCheatsEnabled(),
 			"disabledTemplates": cmpPlayer.GetDisabledTemplates(),
 			"hasSharedDropsites": cmpPlayer.HasSharedDropsites(),
+			"hasSharedLos": cmpPlayer.HasSharedLos(),
 			"phase": phase,
 			"isAlly": allies,
 			"isMutualAlly": mutualAllies,
@@ -169,7 +170,7 @@ GuiInterface.prototype.GetSimulationState = function(player)
  * Note: Amongst statistics, the team exploration map percentage is computed from
  * scratch, so the extended simulation state should not be requested too often.
  */
-GuiInterface.prototype.GetExtendedSimulationState = function(player)
+GuiInterface.prototype.GetExtendedSimulationState = function()
 {
 	// Get basic simulation info
 	let ret = this.GetSimulationState();
@@ -196,7 +197,7 @@ GuiInterface.prototype.GetRenamedEntities = function(player)
 		return this.renamedEntities;
 };
 
-GuiInterface.prototype.ClearRenamedEntities = function(player)
+GuiInterface.prototype.ClearRenamedEntities = function()
 {
 	this.renamedEntities = [];
 	this.miragedEntities = [];
@@ -234,8 +235,10 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 		"garrisonHolder": null,
 		"gate": null,
 		"guard": null,
+		"market": null,
 		"mirage": null,
 		"pack": null,
+		"upgrade" : null,
 		"player": -1,
 		"position": null,
 		"production": null,
@@ -270,9 +273,9 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 	let cmpHealth = QueryMiragedInterface(ent, IID_Health);
 	if (cmpHealth)
 	{
-		ret.hitpoints = Math.ceil(cmpHealth.GetHitpoints());
+		ret.hitpoints = cmpHealth.GetHitpoints();
 		ret.maxHitpoints = cmpHealth.GetMaxHitpoints();
-		ret.needsRepair = cmpHealth.IsRepairable() && (cmpHealth.GetHitpoints() < cmpHealth.GetMaxHitpoints());
+		ret.needsRepair = cmpHealth.IsRepairable() && cmpHealth.GetHitpoints() < cmpHealth.GetMaxHitpoints();
 		ret.needsHeal = !cmpHealth.IsUnhealable();
 		ret.canDelete = !cmpHealth.IsUndeletable();
 	}
@@ -288,6 +291,13 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 	if (cmpBuilder)
 		ret.builder = true;
 
+	let cmpMarket = QueryMiragedInterface(ent, IID_Market);
+	if (cmpMarket)
+		ret.market = {
+			"land": cmpMarket.HasType("land"),
+			"naval": cmpMarket.HasType("naval"),
+		};
+
 	let cmpPack = Engine.QueryInterface(ent, IID_Pack);
 	if (cmpPack)
 		ret.pack = {
@@ -295,29 +305,34 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 			"progress": cmpPack.GetProgress(),
 		};
 
+	var cmpUpgrade = Engine.QueryInterface(ent, IID_Upgrade);
+	if (cmpUpgrade)
+		ret.upgrade = {
+			"upgrades" : cmpUpgrade.GetUpgrades(),
+			"progress": cmpUpgrade.GetProgress(),
+			"template": cmpUpgrade.GetUpgradingTo()
+		};
+
 	let cmpProductionQueue = Engine.QueryInterface(ent, IID_ProductionQueue);
 	if (cmpProductionQueue)
 		ret.production = {
 			"entities": cmpProductionQueue.GetEntitiesList(),
 			"technologies": cmpProductionQueue.GetTechnologiesList(),
-			"queue": cmpProductionQueue.GetQueue(),
+			"techCostMultiplier": cmpProductionQueue.GetTechCostMultiplier(),
+			"queue": cmpProductionQueue.GetQueue()
 		};
 
 	let cmpTrader = Engine.QueryInterface(ent, IID_Trader);
 	if (cmpTrader)
 		ret.trader = {
-			"goods": cmpTrader.GetGoods(),
-			"requiredGoods": cmpTrader.GetRequiredGoods()
+			"goods": cmpTrader.GetGoods()
 		};
 
 	let cmpFogging = Engine.QueryInterface(ent, IID_Fogging);
 	if (cmpFogging)
-	{
-		if (cmpFogging.IsMiraged(player))
-			ret.fogging = { "mirage": cmpFogging.GetMirage(player) };
-		else
-			ret.fogging = { "mirage": null };
-	}
+		ret.fogging = {
+			"mirage": cmpFogging.IsMiraged(player) ? cmpFogging.GetMirage(player) : null
+		};
 
 	let cmpFoundation = QueryMiragedInterface(ent, IID_Foundation);
 	if (cmpFoundation)
@@ -342,6 +357,7 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 	if (cmpGarrisonHolder)
 		ret.garrisonHolder = {
 			"entities": cmpGarrisonHolder.GetEntities(),
+			"buffHeal": cmpGarrisonHolder.GetHealRate(),
 			"allowedClasses": cmpGarrisonHolder.GetAllowedClasses(),
 			"capacity": cmpGarrisonHolder.GetCapacity(),
 			"garrisonedEntitiesCount": cmpGarrisonHolder.GetGarrisonedEntitiesCount()
@@ -404,7 +420,8 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 		"attack": null,
 		"barterMarket": null,
 		"buildingAI": null,
-		"healer": null,
+		"heal": null,
+		"loot": null,
 		"obstruction": null,
 		"turretParent":null,
 		"promotion": null,
@@ -413,6 +430,7 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 		"resourceDropsite": null,
 		"resourceGatherRates": null,
 		"resourceSupply": null,
+		"speed": null,
 	};
 
 	let cmpIdentity = Engine.QueryInterface(ent, IID_Identity);
@@ -426,12 +444,16 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 		for (let type of types)
 		{
 			ret.attack[type] = cmpAttack.GetAttackStrengths(type);
+			ret.attack[type].splash = cmpAttack.GetSplashDamage(type);
+
 			let range = cmpAttack.GetRange(type);
 			ret.attack[type].minRange = range.min;
 			ret.attack[type].maxRange = range.max;
+
 			let timers = cmpAttack.GetTimers(type);
 			ret.attack[type].prepareTime = timers.prepare;
 			ret.attack[type].repeatTime = timers.repeat;
+
 			if (type != "Ranged")
 			{
 				// not a ranged attack, set some defaults
@@ -441,12 +463,14 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 			}
 
 			ret.attack[type].elevationBonus = range.elevationBonus;
+
 			let cmpPosition = Engine.QueryInterface(ent, IID_Position);
 			let cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
 			let cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
+
 			if (cmpUnitAI && cmpPosition && cmpPosition.IsInWorld())
 			{
-				// For units, take the rage in front of it, no spread. So angle = 0
+				// For units, take the range in front of it, no spread. So angle = 0
 				ret.attack[type].elevationAdaptedRange = cmpRangeManager.GetElevationAdaptedRange(cmpPosition.GetPosition(), cmpPosition.GetRotation(), range.max, range.elevationBonus, 0);
 			}
 			else if(cmpPosition && cmpPosition.IsInWorld())
@@ -474,6 +498,7 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 	if (cmpBuildingAI)
 		ret.buildingAI = {
 			"defaultArrowCount": cmpBuildingAI.GetDefaultArrowCount(),
+			"maxArrowCount": cmpBuildingAI.GetMaxArrowCount(),
 			"garrisonArrowMultiplier": cmpBuildingAI.GetGarrisonArrowMultiplier(),
 			"garrisonArrowClasses": cmpBuildingAI.GetGarrisonArrowClasses(),
 			"arrowCount": cmpBuildingAI.GetArrowCount()
@@ -537,11 +562,34 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 
 	let cmpHeal = Engine.QueryInterface(ent, IID_Heal);
 	if (cmpHeal)
-		ret.healer = {
+		ret.heal = {
+			"hp": cmpHeal.GetHP(),
+			"range": cmpHeal.GetRange().max,
+			"rate": cmpHeal.GetRate(),
 			"unhealableClasses": cmpHeal.GetUnhealableClasses(),
 			"healableClasses": cmpHeal.GetHealableClasses(),
 		};
 
+	let cmpLoot = Engine.QueryInterface(ent, IID_Loot);
+	if (cmpLoot)
+	{
+		let resources = cmpLoot.GetResources();
+		ret.loot = {
+			"xp": cmpLoot.GetXp(),
+			"food": resources.food,
+			"wood": resources.wood,
+			"stone": resources.stone,
+			"metal": resources.metal
+		};
+	}
+
+	let cmpUnitMotion = Engine.QueryInterface(ent, IID_UnitMotion);
+	if (cmpUnitMotion)
+		ret.speed = {
+			"walk": cmpUnitMotion.GetWalkSpeed(),
+			"run": cmpUnitMotion.GetRunSpeed()
+		};
+
 	return ret;
 };
 
@@ -576,13 +624,34 @@ GuiInterface.prototype.GetTemplateData = function(player, extendedName)
 	if (!template)
 		return null;
 
-	return GetTemplateDataHelper(template, player);
+	let aurasTemplate = {};
+
+	if (!template.Auras)
+		return GetTemplateDataHelper(template, player, aurasTemplate);
+
+	// Add aura name and description loaded from JSON file
+	let auraNames = template.Auras._string.split(/\s+/);
+	let cmpDataTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_DataTemplateManager);
+	for (let name of auraNames)
+	{
+		let auraTemplate = cmpDataTemplateManager.GetAuraTemplate(name);
+		if (!auraTemplate)
+		{
+			// The following warning is perhaps useless since it's yet done in DataTemplateManager
+			warn("Tried to get data for invalid aura: " + name);
+			continue;
+		}
+		aurasTemplate[name] = {};
+		aurasTemplate[name].auraName = auraTemplate.auraName || null;
+		aurasTemplate[name].auraDescription = auraTemplate.auraDescription || null;
+	}
+	return GetTemplateDataHelper(template, player, aurasTemplate);
 };
 
 GuiInterface.prototype.GetTechnologyData = function(player, name)
 {
-	let cmpTechnologyTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TechnologyTemplateManager);
-	let template = cmpTechnologyTemplateManager.GetTemplate(name);
+	let cmpDataTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_DataTemplateManager);
+	let template = cmpDataTemplateManager.GetTechnologyTemplate(name);
 
 	if (!template)
 	{
@@ -620,9 +689,9 @@ GuiInterface.prototype.CheckTechnologyRequirements = function(player, data)
 
 // Returns technologies that are being actively researched, along with
 // which entity is researching them and how far along the research is.
-GuiInterface.prototype.GetStartedResearch = function(player, viewedPlayer)
+GuiInterface.prototype.GetStartedResearch = function(player)
 {
-	let cmpTechnologyManager = QueryPlayerIDInterface(viewedPlayer, IID_TechnologyManager);
+	let cmpTechnologyManager = QueryPlayerIDInterface(player, IID_TechnologyManager);
 	if (!cmpTechnologyManager)
 		return {};
 
@@ -640,9 +709,9 @@ GuiInterface.prototype.GetStartedResearch = function(player, viewedPlayer)
 };
 
 // Returns the battle state of the player.
-GuiInterface.prototype.GetBattleState = function(player, viewedPlayer)
+GuiInterface.prototype.GetBattleState = function(player)
 {
-	let cmpBattleDetection = QueryPlayerIDInterface(viewedPlayer, IID_BattleDetection);
+	let cmpBattleDetection = QueryPlayerIDInterface(player, IID_BattleDetection);
 
 	if (!cmpBattleDetection)
 		return false;
@@ -696,11 +765,11 @@ GuiInterface.prototype.DeleteTimeNotification = function(notificationID)
 	this.timeNotifications = this.timeNotifications.filter(n => n.id != notificationID);
 };
 
-GuiInterface.prototype.GetTimeNotifications = function(playerID, viewedPlayer)
+GuiInterface.prototype.GetTimeNotifications = function(player)
 {
 	let time = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer).GetTime();
 	// filter on players and time, since the delete timer might be executed with a delay
-	return this.timeNotifications.filter(n => n.players.indexOf(viewedPlayer) != -1 && n.endTime > time);
+	return this.timeNotifications.filter(n => n.players.indexOf(player) != -1 && n.endTime > time);
 };
 
 GuiInterface.prototype.PushNotification = function(notification)
@@ -790,7 +859,6 @@ GuiInterface.prototype.GetAllBuildableEntities = function(player, cmd)
 
 GuiInterface.prototype.SetSelectionHighlight = function(player, cmd)
 {
-	let cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
 	let playerColors = {}; // cache of owner -> color map
 
 	for each (let ent in cmd.entities)
@@ -809,7 +877,7 @@ GuiInterface.prototype.SetSelectionHighlight = function(player, cmd)
 		if (!color)
 		{
 			color = { "r":1, "g":1, "b":1 };
-			let cmpPlayer = Engine.QueryInterface(cmpPlayerManager.GetPlayerByID(owner), IID_Player);
+			let cmpPlayer = QueryPlayerIDInterface(owner);
 			if (cmpPlayer)
 				color = cmpPlayer.GetColor();
 			playerColors[owner] = color;
@@ -1616,34 +1684,91 @@ GuiInterface.prototype.PlaySound = function(player, data)
 	PlaySound(data.name, data.entity);
 };
 
+/**
+ * Find any idle units.
+ *
+ * @param data.idleClasses		Array of class names to include.
+ * @param data.prevUnit		The previous idle unit, if calling a second time to iterate through units.  May be left undefined.
+ * @param data.limit			The number of idle units to return.  May be left undefined (will return all idle units).
+ * @param data.excludeUnits	Array of units to exclude.
+ *
+ * Returns an array of idle units.
+ * If multiple classes were supplied, and multiple items will be returned, the items will be sorted by class.
+ */
 GuiInterface.prototype.FindIdleUnits = function(player, data)
 {
-	let cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
-	let playerEntities = cmpRangeManager.GetEntitiesByPlayer(data.viewedPlayer).filter(entity => {
-
-		let cmpUnitAI = Engine.QueryInterface(entity, IID_UnitAI);
-		if (!cmpUnitAI || !cmpUnitAI.IsIdle() || cmpUnitAI.IsGarrisoned())
-			return false;
-
-		let cmpIdentity = Engine.QueryInterface(entity, IID_Identity);
-		if (!cmpIdentity || !cmpIdentity.HasClass(data.idleClass))
-			return false;
-
-		return true;
-	});
-
 	let idleUnits = [];
-
-	for (let ent of playerEntities)
+	// The general case is that only the 'first' idle unit is required; filtering would examine every unit.
+	// This loop imitates a grouping/aggregation on the first matching idle class.
+	let cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
+	for (let entity of cmpRangeManager.GetEntitiesByPlayer(player))
 	{
-		if (ent <= data.prevUnit|0 || data.excludeUnits.indexOf(ent) > -1)
+		let filtered = this.IdleUnitFilter(entity, data.idleClasses, data.excludeUnits);
+		if (!filtered.idle)
 			continue;
-		idleUnits.push(ent);
-		if (data.limit && idleUnits.length >= data.limit)
-			break;
+
+		// If the entity is in the 'current' (first, 0) bucket on a resumed search, it must be after the "previous" unit, if any.
+		// By adding to the 'end', there is no pause if the series of units loops.
+		var bucket = filtered.bucket;
+		if(bucket == 0 && data.prevUnit && entity <= data.prevUnit)
+			bucket = data.idleClasses.length;
+
+		if (!idleUnits[bucket])
+			idleUnits[bucket] = [];
+		idleUnits[bucket].push(entity);
+
+		// If enough units have been collected in the first bucket, go ahead and return them.
+		if (data.limit && bucket == 0 && idleUnits[0].length == data.limit)
+			return idleUnits[0];
 	}
 
-	return idleUnits;
+	let reduced = idleUnits.reduce((prev, curr) => prev.concat(curr), []);
+	if (data.limit && reduced.length > data.limit)
+		return reduced.slice(0, data.limit);
+
+	return reduced;
+};
+
+/**
+ * Discover if the player has idle units.
+ *
+ * @param data.idleClasses	Array of class names to include.
+ * @param data.excludeUnits	Array of units to exclude.
+ *
+ * Returns a boolean of whether the player has any idle units
+ */
+GuiInterface.prototype.HasIdleUnits = function(player, data)
+{
+	let cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
+	return cmpRangeManager.GetEntitiesByPlayer(player).some(unit => this.IdleUnitFilter(unit, data.idleClasses, data.excludeUnits).idle);
+};
+
+/**
+ * Whether to filter an idle unit
+ *
+ * @param unit			The unit to filter.
+ * @param idleclasses	Array of class names to include.
+ * @param excludeUnits	Array of units to exclude.
+ *
+ * Returns an object with the following fields:
+ * 	- idle - true if the unit is considered idle by the filter, false otherwise.
+ * 	- bucket - if idle, set to the index of the first matching idle class, undefined otherwise.
+ */
+GuiInterface.prototype.IdleUnitFilter = function(unit, idleClasses, excludeUnits)
+{
+	let cmpUnitAI = Engine.QueryInterface(unit, IID_UnitAI);
+	if (!cmpUnitAI || !cmpUnitAI.IsIdle() || cmpUnitAI.IsGarrisoned())
+		return { "idle": false };
+
+	let cmpIdentity = Engine.QueryInterface(unit, IID_Identity);
+	if(!cmpIdentity)
+		return { "idle": false };
+
+	let bucket = idleClasses.findIndex(elem => cmpIdentity.HasClass(elem));
+	if (bucket == -1 || excludeUnits.indexOf(unit) > -1)
+		return { "idle": false };
+
+	return { "idle": true, "bucket": bucket };
 };
 
 GuiInterface.prototype.GetTradingRouteGain = function(player, data)
@@ -1778,10 +1903,10 @@ GuiInterface.prototype.SetRangeDebugOverlay = function(player, enabled)
 	Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager).SetDebugOverlay(enabled);
 };
 
-GuiInterface.prototype.GetTraderNumber = function(player, viewedPlayer)
+GuiInterface.prototype.GetTraderNumber = function(player)
 {
 	let cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
-	let traders = cmpRangeManager.GetEntitiesByPlayer(viewedPlayer).filter(e => Engine.QueryInterface(e, IID_Trader));
+	let traders = cmpRangeManager.GetEntitiesByPlayer(player).filter(e => Engine.QueryInterface(e, IID_Trader));
 
 	let landTrader = { "total": 0, "trading": 0, "garrisoned": 0 };
 	let shipTrader = { "total": 0, "trading": 0 };
@@ -1817,9 +1942,9 @@ GuiInterface.prototype.GetTraderNumber = function(player, viewedPlayer)
 	return { "landTrader": landTrader, "shipTrader": shipTrader };
 };
 
-GuiInterface.prototype.GetTradingGoods = function(player, viewedPlayer)
+GuiInterface.prototype.GetTradingGoods = function(player)
 {
-	return QueryPlayerIDInterface(viewedPlayer).GetTradingGoods();
+	return QueryPlayerIDInterface(player).GetTradingGoods();
 };
 
 GuiInterface.prototype.OnGlobalEntityRenamed = function(msg)
@@ -1870,6 +1995,7 @@ let exposedFunctions = {
 	"GetFoundationSnapData": 1,
 	"PlaySound": 1,
 	"FindIdleUnits": 1,
+	"HasIdleUnits": 1,
 	"GetTradingRouteGain": 1,
 	"GetTradingDetails": 1,
 	"CanCapture": 1,
