@@ -1,14 +1,15 @@
 function TerritoryDecay() {}
 
 TerritoryDecay.prototype.Schema =
-	"<element name='DecayRate' a:help='Decay rate in hitpoints per second'>" +
-		"<ref name='positiveDecimal'/>" +
+	"<element name='DecayRate' a:help='Decay rate in capture points per second'>" +
+		"<choice><ref name='positiveDecimal'/><value>Infinity</value></choice>" +
 	"</element>";
 
 TerritoryDecay.prototype.Init = function()
 {
 	this.decaying = false;
 	this.connectedNeighbours = [];
+	this.territoryOwnership = !isFinite(+this.template.DecayRate);
 };
 
 TerritoryDecay.prototype.IsConnected = function()
@@ -47,9 +48,13 @@ TerritoryDecay.prototype.IsConnected = function()
 
 	for (var i = 1; i < numPlayers; ++i)
 		if (this.connectedNeighbours[i] > 0 && cmpPlayer.IsMutualAlly(i))
-			return true; // don't decay if connected to a connected ally
+		{
+			// don't decay if connected to a connected ally; disable blinking
+			cmpTerritoryManager.SetTerritoryBlinking(pos.x, pos.y, false);
+			return true;
+		}
 
-	cmpTerritoryManager.SetTerritoryBlinking(pos.x, pos.y);
+	cmpTerritoryManager.SetTerritoryBlinking(pos.x, pos.y, true);
 	return false;
 };
 
@@ -84,21 +89,52 @@ TerritoryDecay.prototype.UpdateDecayState = function()
 	Engine.PostMessage(this.entity, MT_TerritoryDecayChanged, { "entity": this.entity, "to": decaying, "rate": this.GetDecayRate() });
 };
 
+TerritoryDecay.prototype.UpdateOwner = function()
+{
+	let cmpOwnership = Engine.QueryInterface(this.entity, IID_Ownership);
+	let cmpPosition = Engine.QueryInterface(this.entity, IID_Position);
+	if (!cmpOwnership || !cmpPosition || !cmpPosition.IsInWorld())
+		return;
+	let cmpTerritoryManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TerritoryManager);
+	let pos = cmpPosition.GetPosition2D();
+	let tileOwner = cmpTerritoryManager.GetOwner(pos.x, pos.y);
+	if (tileOwner != cmpOwnership.GetOwner())
+		cmpOwnership.SetOwner(tileOwner);
+};
+
 TerritoryDecay.prototype.OnTerritoriesChanged = function(msg)
 {
-	this.UpdateDecayState();
+	if (this.territoryOwnership)
+		this.UpdateOwner();
+	else
+		this.UpdateDecayState();
 };
 
 TerritoryDecay.prototype.OnTerritoryPositionChanged = function(msg)
 {
-	this.UpdateDecayState();
+	if (this.territoryOwnership)
+		this.UpdateOwner();
+	else
+		this.UpdateDecayState();
+};
+
+TerritoryDecay.prototype.OnDiplomacyChanged = function(msg)
+{
+	// Can change the connectedness of certain areas
+	if (!this.territoryOwnership)
+		this.UpdateDecayState();
 };
 
 TerritoryDecay.prototype.OnOwnershipChanged = function(msg)
 {
 	// if it influences the territory, wait until we get a TerritoriesChanged message
-	if (!Engine.QueryInterface(this.entity, IID_TerritoryInfluence))
+	if (!this.territoryOwnership && !Engine.QueryInterface(this.entity, IID_TerritoryInfluence))
 		this.UpdateDecayState();
 };
 
+TerritoryDecay.prototype.HasTerritoryOwnership = function()
+{
+	return this.territoryOwnership;
+};
+
 Engine.RegisterComponentType(IID_TerritoryDecay, "TerritoryDecay", TerritoryDecay);
